import Vue from 'vue';

var script = Vue.extend({
  name: 'VTextareaHighlighter',

  data() {
    return {
      text: '',
      backdropStyle: {}
    };
  },

  computed: {
    lightedHtml() {
      return this.$data.text.replace(/\n$/g, '\n\n').replace(/[A-Z].*?\b/g, '<mark>$&</mark>');
    }

  },
  methods: {
    getElTextarea() {
      var _this$$slots$default, _this$$slots$default$;

      return (_this$$slots$default = this.$slots.default) === null || _this$$slots$default === void 0 ? void 0 : (_this$$slots$default$ = _this$$slots$default[0]) === null || _this$$slots$default$ === void 0 ? void 0 : _this$$slots$default$.elm;
    },

    getElBackdrop() {
      return this.$refs.backdrop;
    },

    alignBackdropStyle() {
      const {
        padding,
        font,
        letterSpacing,
        border
      } = window.getComputedStyle(this.getElTextarea());
      this.backdropStyle = {
        padding,
        font,
        letterSpacing,
        border
      };
    },

    handleScroll() {
      this.getElBackdrop().scrollTop = this.getElTextarea().scrollTop;
      this.getElBackdrop().scrollLeft = this.getElTextarea().scrollLeft;
    },

    handleInput(e) {
      this.$data.text = e.target.value;
    }

  },

  mounted() {
    this.alignBackdropStyle();
    this.$data.text = this.getElTextarea().value;
    this.getElTextarea().addEventListener('scroll', this.handleScroll);
    this.getElTextarea().addEventListener('input', this.handleInput);
  },

  beforeDestroy() {
    this.getElTextarea().removeEventListener('scroll', this.handleScroll);
    this.getElTextarea().removeEventListener('input', this.handleInput);
  }

});

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

const isOldIE = typeof navigator !== 'undefined' &&
    /msie [6-9]\\b/.test(navigator.userAgent.toLowerCase());
function createInjector(context) {
    return (id, style) => addStyle(id, style);
}
let HEAD;
const styles = {};
function addStyle(id, css) {
    const group = isOldIE ? css.media || 'default' : id;
    const style = styles[group] || (styles[group] = { ids: new Set(), styles: [] });
    if (!style.ids.has(id)) {
        style.ids.add(id);
        let code = css.source;
        if (css.map) {
            // https://developer.chrome.com/devtools/docs/javascript-debugging
            // this makes source maps inside style tags work properly in Chrome
            code += '\n/*# sourceURL=' + css.map.sources[0] + ' */';
            // http://stackoverflow.com/a/26603875
            code +=
                '\n/*# sourceMappingURL=data:application/json;base64,' +
                    btoa(unescape(encodeURIComponent(JSON.stringify(css.map)))) +
                    ' */';
        }
        if (!style.element) {
            style.element = document.createElement('style');
            style.element.type = 'text/css';
            if (css.media)
                style.element.setAttribute('media', css.media);
            if (HEAD === undefined) {
                HEAD = document.head || document.getElementsByTagName('head')[0];
            }
            HEAD.appendChild(style.element);
        }
        if ('styleSheet' in style.element) {
            style.styles.push(code);
            style.element.styleSheet.cssText = style.styles
                .filter(Boolean)
                .join('\n');
        }
        else {
            const index = style.ids.size - 1;
            const textNode = document.createTextNode(code);
            const nodes = style.element.childNodes;
            if (nodes[index])
                style.element.removeChild(nodes[index]);
            if (nodes.length)
                style.element.insertBefore(textNode, nodes[index]);
            else
                style.element.appendChild(textNode);
        }
    }
}

/* script */
const __vue_script__ = script;
/* template */

var __vue_render__ = function () {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    staticClass: "container"
  }, [_c('div', {
    ref: "backdrop",
    staticClass: "backdrop",
    style: _vm.backdropStyle
  }, [_c('div', {
    ref: "highlights",
    staticClass: "highlights",
    domProps: {
      "innerHTML": _vm._s(_vm.lightedHtml)
    }
  })]), _vm._v(" "), _vm._t("default")], 2);
};

var __vue_staticRenderFns__ = [];
/* style */

const __vue_inject_styles__ = function (inject) {
  if (!inject) return;
  inject("data-v-c14bb2e6_0", {
    source: ".backdrop[data-v-c14bb2e6],.container[data-v-c14bb2e6],textarea[data-v-c14bb2e6]{width:460px;height:180px}.container[data-v-c14bb2e6]{display:block;margin:0 auto}.backdrop[data-v-c14bb2e6]{position:absolute;z-index:1;overflow:auto;pointer-events:none;transition:transform 1s}.highlights[data-v-c14bb2e6]{white-space:pre-wrap;word-wrap:break-word;color:transparent}textarea[data-v-c14bb2e6]{display:block;position:absolute;z-index:2;margin:0;border-radius:0;background-color:transparent;overflow:auto;resize:none}mark[data-v-c14bb2e6]{border-radius:3px;color:transparent;background-color:#b1d5e5}",
    map: undefined,
    media: undefined
  });
};
/* scoped */


const __vue_scope_id__ = "data-v-c14bb2e6";
/* module identifier */

const __vue_module_identifier__ = undefined;
/* functional template */

const __vue_is_functional_template__ = false;
/* style inject SSR */

/* style inject shadow dom */

const __vue_component__ = /*#__PURE__*/normalizeComponent({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__, __vue_module_identifier__, false, createInjector, undefined, undefined);

var component = __vue_component__;

// Import vue component
// IIFE injects install function into component, allowing component
// to be registered via Vue.use() as well as Vue.component(),

var entry_esm = /*#__PURE__*/(() => {
  // Assign InstallableComponent type
  const installable = component; // Attach install function executed by Vue.use()

  installable.install = Vue => {
    Vue.component('VTextareaHighlighter', installable);
  };

  return installable;
})(); // It's possible to expose named exports when writing components that can
// also be used as directives, etc. - eg. import { RollupDemoDirective } from 'rollup-demo';
// export const RollupDemoDirective = directive;

export { entry_esm as default };
